/*
   File:       CONNECT.CPP
   Full name:  Powersim Connection.
   Language:   Microsoft Visual C++
   Author:     Arne Krkenes (AK), Olav Svennevik (OS), Bjrn Arild W. Baugst (BB)
   Copyright:  1995-1996, ModellData
   Changes:    When     Who   What
               950316   AK    Created.
               950809   OS    Added NotifyOpenModel(). Stopping simulation in ~CPsConnection().
               960117   AK    Changed name from CModel to CPsConnection
               960129   BB    Added functions for new features of PSAPI 
*/


#include "stdafx.h"
#include "connect.h"

#ifndef _WIN32   
#include <float.h>
#endif

// Inline helpers

#ifdef _WIN32
#define _fmemset memset
#endif

inline double __cdecl DoubleNAN() {
   double D;
   _fmemset(&D, 0xff, sizeof(D));
   ((WORD *)&D)[3] = 0xFFFF;      // Bug in MS-C, 00FF00FF00FF00FF...
   return D;
}


inline BOOL IsNAN(double D) {
   register WORD c = ((WORD *)&D)[3];

   return ((c & 0x7FF0) == 0x7FF0) && (c & 0x000F);
}


// Construction/destruction


CPsConnection::CPsConnection() {
   lstrcpy(m_szModelPath, "") ;
   m_hConnection = 0 ;
   m_hwndApp = NULL ;
   m_bPaused = FALSE ;
   m_State = SimStopped ;
   m_dbStartTime = DoubleNAN() ;
   m_dbStopTime = DoubleNAN() ;
   m_dbTimeStep = DoubleNAN() ;
   m_dbTimeDelay = DoubleNAN();   // BB960126: Added
   m_dbPauseTime = DoubleNAN();   // BB960301: Added
} // CPsConnection::CPsConnection


CPsConnection::~CPsConnection() {
   Close() ;
} // CPsConnection::~CPsConnection


// Connection


void CPsConnection::SetTimeSpec(BOOL bUpdatePause) {
/* Purpose:    Set the time specs of the model.
*/
   if (!IsNAN(m_dbStartTime))
      PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_STARTTIME, m_dbStartTime) ;
   if (!IsNAN(m_dbStopTime))
      PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_STOPTIME, m_dbStopTime) ;
   if (!IsNAN(m_dbTimeStep))
      PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_TIMESTEP, m_dbTimeStep) ;
   if (!IsNAN(m_dbTimeDelay))   // BB960126: Added
      PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_TIMEDELAY, m_dbTimeDelay) ;
   if (!IsNAN(m_dbPauseTime))   // BB960301: Added
      PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_PAUSEATTIME, m_dbPauseTime) ;
   if (bUpdatePause)
      PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_PAUSE, (m_bPaused ? 1.0 : 0.0)) ;
} // CPsConnection::SetTimeSpec


void CPsConnection::UpdateCtrlParams() {
/* Purpose:    Get control parameters from the model.
*/
   double dbValue ;
   PSAPI_GetControlValue(m_hConnection, PSAPI_CTL_PLAY, &dbValue) ;
   m_State = (dbValue != 0.0 ? SimRunning : SimStopped) ;

   SetTimeSpec(TRUE) ;
   
   if (IsNAN(m_dbTimeDelay))   // BB960126: Added
      PSAPI_GetControlValue(m_hConnection, PSAPI_CTL_TIMEDELAY, &m_dbTimeDelay) ;
   if (IsNAN(m_dbTimeStep))
      PSAPI_GetControlValue(m_hConnection, PSAPI_CTL_TIMESTEP, &m_dbTimeStep) ;
   if (IsNAN(m_dbStopTime))
      PSAPI_GetControlValue(m_hConnection, PSAPI_CTL_STOPTIME, &m_dbStopTime) ;
   if (IsNAN(m_dbStartTime))
      PSAPI_GetControlValue(m_hConnection, PSAPI_CTL_STARTTIME, &m_dbStartTime) ;
   PSAPI_GetControlValue(m_hConnection, PSAPI_CTL_TIME, &m_dbTime) ;
} // CPsConnection::UpdateCtrlParams


BOOL CPsConnection::VerifyOpen(BOOL bBrowse) {
/* Purpose:    Verify that the model is opened by Powersim. If the model was not opened,
               this function will try to open it.
   Return:     'FALSE' if the model is not open, and it could not be opened.
*/
   if (!m_hConnection && m_hwndApp) {
      m_hConnection = PSAPI_OpenConnection(m_szModelPath, 
         bBrowse ? m_szModelPath : NULL, m_hwndApp, TRUE, this) ;
      if (m_hConnection)
      {
         UpdateCtrlParams() ;
         NotifyOpenModel();  // OS950809
      }
   } // if
   return (m_hConnection != NULL) ;
} // CPsConnection::VerifyOpen


BOOL CPsConnection::Open(LPCSTR lpszName, HWND hwndApp, BOOL bBrowse) {
/* Purpose:    Open the model named 'lpszName'.
   In:         'lpszName' - an absolute path of the model to be opened, 
                  or a path relative to the current path or a path relative
                  to the executable given by 'hinstModule'.
               'hwndApp' - a window in the application that is to receive notification
                  messages. If NULL connection will not be opened, but
                  model name will be set.
               'bBrowse' - browse if file not found ?
    Return:    Whether or not Powersim was able to open the model.
*/
   if (lstrcmpi(m_szModelPath, lpszName) || m_hwndApp != hwndApp)
      Close() ;
   lstrcpy(m_szModelPath, lpszName) ;
   m_hwndApp = hwndApp ;
   return VerifyOpen(bBrowse) ;
} // CPsConnection::Open


void CPsConnection::Close() {
/* Purpose:    Close the model in Powersim, and delete the references to the model in
               this 'CPsConnection' object.
*/
   if (m_hConnection) {
      if (IsRunning()) // AK951019 Added if check to prevent stopping disconnected models.
         Play(FALSE);  

      HCONNECTION hConnection = m_hConnection ;
      m_hConnection = 0 ;
      PSAPI_CloseConnection(hConnection) ;
   } // if
   
   lstrcpy(m_szModelPath, "") ;
   m_hwndApp = NULL ;
} // CPsConnection::Close


// Parameter observers

int CPsConnection::GetParCount() const {
/* Return:     The number of parameters.
*/
   return PSAPI_GetParCount(m_hConnection);
} // CPsConnection::GetParCount


LPSTR CPsConnection::GetParName(int iPar, LPSTR lpszName, UINT uiSizeOfName) const {
/* Purpose:    Get the name of a parameter.
   In:         'iPar' - the index of the parameter.
               'uiSizeOfName' - the number of bytes in the 'lpszName' buffer.
   Out:        'lpszName[]' - the buffer is filled with the parameter name.
   Return:     'lpszName'.
*/
   ASSERT(iPar >= 0) ;
   LPSTR lpszRet = PSAPI_GetParName(m_hConnection, iPar, lpszName, uiSizeOfName) ;
   return lpszName ;
} // CPsConnection::GetParName


int CPsConnection::SearchPar(LPCSTR lpszParName) const {
/* Purpose:    Get the index of a parameter with a given name.
   In:         'lpszParName' - the name to be searched for.
   Return:     The index of the parameter to be searched for, or -1 if the parameter
               name was not found.
*/
   return PSAPI_SearchPar(m_hConnection, lpszParName);
} // CPsConnection::SearchPar


LPSTR CPsConnection::GetParUnit(int iPar, LPSTR lpszUnit, UINT uiSizeOfUnit) const {
/* Purpose:    Get the unit of a parameter.
   In:         'iPar' - the index of the parameter.
               'uiSizeOfUnit' - the number of bytes in the 'lpszUnit' buffer.
   Out:        'lpszUnit[]' - the buffer is filled with the unit name.
   Return:     'lpszName'.
   Changes:    When     Who   What
               960126   BB    Created
*/
   ASSERT(iPar >= 0);
   LPSTR lpszRet = PSAPI_GetParUnit(m_hConnection, iPar, lpszUnit, uiSizeOfUnit);
   return lpszUnit;
} // CPsConnection::GetParUnit


BOOL CPsConnection::GetParScale(int iPar, double FAR *pdbMin, double FAR *pdbMax) const {
/* Purpose:    Get the min and max scale values of a parameter.
   In:         'iPar' - the index of the parameter.
   Out:        '*pdbMin' - the minimum scale of the parameter
               '*pdbMin' - the maximum scale of teh parameter
   Return:     TRUE if the operation succeeded
   Changes:    When     Who   What
               960126   BB    Created
*/
   ASSERT(iPar >= 0);
   return PSAPI_GetParScale(m_hConnection, iPar, pdbMin, pdbMax);
} // CPsConnection::GetParScale


// Handling parameters.


int CPsConnection::AppendPar(LPCSTR lpszParName) {
/* Purpose:    Append a parameter with a given name.
   In:         The name of the parameter to append to the parameter list.
   Return:     The index of the parameter in the parameter list. -1 is
               returned if there are no variables in the model with that 
               name, or if there is a memory problem.
*/
   return (VerifyOpen() ? PSAPI_AppendPar(m_hConnection, lpszParName) : -1) ;
} // CPsConnection::AppendPar


BOOL CPsConnection::RemoveParAt(int iPar) const {
/* Purpose:    Remove a parameter from the parameter list.
   In:         'iPar' - the index of the parameter to be removed.
   Note:       Does not remove the variable from the model.
   Changes:    When     Who   What
               960124   BB    Changed return value to BOOL.
*/
   ASSERT(iPar >= 0) ;
   return PSAPI_RemoveParAt(m_hConnection, iPar) ;
} // CPsConnection::RemoveParAt


BOOL CPsConnection::RemoveAllPar() const {
/* Purpose:    Remove all parameters from the parameter list.
   Changes:    When     Who   What
               960124   BB    Changed return value to BOOL.
*/
   return PSAPI_RemoveAllPar(m_hConnection) ;
} // CPsConnection::RemoveAllPar


// Getting and setting values of parameters


BOOL CPsConnection::GetParValue(int iPar, double *pdbValue) const {
/* Purpose:    Get the value of a parameter.
   In:         'iPar' - the index of the parameter to be retrieved.
   Out:        'pdbValue' - the value of the parameter.
   Return:     TRUE if the operation succeeded.
*/
   ASSERT(iPar >= 0) ;
   BOOL bRet ;
   if (m_ValueTab)
   {
      bRet = (iPar>=0 && iPar < m_cValuesInTab) ;
      if (bRet)
         *pdbValue = m_ValueTab[iPar] ;
      else
         *pdbValue = DoubleNAN() ;
   }
   else
   {
      bRet = PSAPI_GetParValue(m_hConnection, iPar, pdbValue) ;
   }
   return bRet ;
} // CPsConnection::GetParValue


BOOL CPsConnection::SetParValue(int iPar, double dbValue) {
/* Purpose:    Set the value of a parameter.
   In:         'iPar' - the index of the parameter to set the value of.
               'dbValue' - the value to assign to the parameter.
   Return:     Whether or not the operation succeeded.
*/
   ASSERT(iPar >= 0) ;
   return PSAPI_SetParValue(m_hConnection, iPar, dbValue) ;
} // CPsConnection::SetParValue


// Ranges


BOOL CPsConnection::BeginChangeRanges() {
/* Purpose:    This function must be called before any ranges are changed.
*/
   return VerifyOpen() && PSAPI_BeginChangeRanges(m_hConnection) ;
} // CPsConnection::BeginChangeRanges


BOOL CPsConnection::EndChangeRanges() {
/* Purpose:    This function must be called when finished changing the ranges.
*/
   return VerifyOpen() && PSAPI_EndChangeRanges(m_hConnection) ;
} // CPsConnection::EndChangeRanges


BOOL CPsConnection::SetRange(LPCSTR lpszRangeName, short nFirst, short nLast) {
/* Purpose:    Set the interval of a range.
   In:         'lpszRangeName' - the name of the range to modify.
               'nFirst' - the lower value in the range.
               'nLast' - the upper value in the range.
*/
   return VerifyOpen() && PSAPI_SetRange(m_hConnection, lpszRangeName, nFirst, nLast) ;
} // CPsConnection::SetRange


// Simulation


BOOL CPsConnection::Play(BOOL bPlayOn) {
/* Purpose:    Start or stop the simulation.
   In:         'bPlayOn' - 'TRUE' to start, and 'FALSE' to stop the simulation.
*/
   BOOL bOk = VerifyOpen() ;
   if (bOk) {
      bOk = PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_PLAY, (bPlayOn ? 1.0 : 0.0)) ;
   } // if
   return bOk ;
} // CPsConnection::Play


BOOL CPsConnection::PlayStep(long cSteps) {
/* Purpose:    Play the number of steps.
   In:         'cSteps' - teh number of steps to play
*/
   BOOL bOk = VerifyOpen() ;
   if (bOk) {
      bOk = PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_PLAYSTEP, (double)cSteps) ;
   } // if
   return bOk ;
} // CPsConnection::PlayStep

BOOL CPsConnection::Pause(BOOL bPauseOn) {
/* Purpose:    Pause on/off.
   In:         'bPauseOn' - TRUE if the simulation is to be paused, or FALSE if not.
*/
   BOOL bOk = (VerifyOpen()) ;
   if (bOk)
      bOk = PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_PAUSE, (bPauseOn ? 1.0 : 0.0)) ;
   return bOk ;
} // CPsConnection::Pause


BOOL CPsConnection::IsRunning() const {
/* Purpose:    TRUE if the simulation is running.
*/
   return (m_State == SimRunning) ;
} // CPsConnection::IsRunning


BOOL CPsConnection::IsPaused() const {
/* Purpose:    TRUE if the simulation is paused.
*/
   return m_bPaused ;
} // CPsConnection::IsPaused

BOOL CPsConnection::EnableWantPauseNotifications(BOOL bEnable) {   // BB960301: Added
/* Purpose:    Enable/disable WantPause notifications
   In:         bEnable - Enable or disable
*/
   return PSAPI_EnableWantPauseNotifications(m_hConnection, bEnable);
} // CPsConnection::EnableWantPauseNotifications

void CPsConnection::SetStartTime(double dbStartTime) {
/* Purpose:    Set the start time of the model.
   In:         The new start time.
   Note:       The start time won't be used before the next run.
*/
   m_dbStartTime = dbStartTime ;
   //PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_STARTTIME, m_dbStartTime) ;
} // CPsConnection::SetStartTime


void CPsConnection::SetStopTime(double dbStopTime) {
/* Purpose:    Set the stop time of the model.
   In:         The new stop time.
   Note:       The stop time won't be used before the next run.
*/
   m_dbStopTime = dbStopTime ;
   //PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_STOPTIME, m_dbStopTime) ;
} // CPsConnection::SetStopTime


void CPsConnection::SetTimeStep(double dbTimeStep) {
/* Purpose:    Set the time step of the model.
   In:         The new time step.
   Note:       The time step won't be used before the next run.
*/
   m_dbTimeStep = dbTimeStep ;
   //PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_TIMESTEP, m_dbTimeStep) ;
} // CPsConnection::SetTimeStep


void CPsConnection::SetTimeDelay(double dbTimeDelay) {
/* Purpose:    Set the delay time of the model.
   In:         The new delay time.
   Note:       The delay time won't be used before the next run.
   Changes:    When     Who   What
               960126   BB    Created
*/
   m_dbTimeDelay = dbTimeDelay ;
   PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_TIMEDELAY, m_dbTimeDelay) ;
} // CPsConnection::SetTimeDelay


void CPsConnection::SetPauseAtTime(double dbPauseTime) {
/* Purpose:    Set the pause at time for the model.
   In:         The time to pause.
   Changes:    When     Who   What
               960301   BB    Created
*/
   m_dbPauseTime = dbPauseTime;
   PSAPI_SetControlValue(m_hConnection, PSAPI_CTL_PAUSEATTIME, m_dbPauseTime) ;
} // CPsConnection::SetPauseAtTime

// Notification


LRESULT CPsConnection::Notify(LPARAM lParam) {
/* Purpose:    This function must be called in response to the notification message
               sent to the window supplied to the Open function.
*/
   LPCOPYDATASTRUCT lpcds = (LPCOPYDATASTRUCT)lParam ;
   LPNOTIFICATIONDATASTRUCT lpnds = (LPNOTIFICATIONDATASTRUCT)lpcds->lpData ;
   CPsConnection* pConnection = (CPsConnection*)lpnds->lpUserData ;
   LRESULT lrRet = FALSE ;
   #ifndef _WIN32   
   _clear87();
   #endif
   switch ((UINT)lpcds->dwData) {
   case PSAPIN_CLOSEMODEL:
      pConnection->m_hConnection = 0 ;
      pConnection->NotifyCloseModel() ;
      break ;
   case PSAPIN_UPDATEVALUES: {
      pConnection->m_dbTime = lpnds->uvs.dbTime ;
      pConnection->m_cValuesInTab = (int)((lpcds->cbData-16)/sizeof(double)) ;
      if (pConnection->m_cValuesInTab)
         pConnection->m_ValueTab = &lpnds->uvs.dbValueTab[0] ;
      pConnection->NotifyUpdateValues((BOOL) lpnds->uvs.bIsFinal) ;
      pConnection->m_cValuesInTab = 0 ;
      pConnection->m_ValueTab = NULL ;
      break ;
   } // case
   case PSAPIN_DELETEPAR:
      ASSERT((int) lpnds->lIndex >= 0) ;
      pConnection->NotifyParamDeleted((int) lpnds->lIndex) ;
      break ;
   case PSAPIN_PREBEGINSIM:
      pConnection->SetTimeSpec(FALSE) ;   // Ensure that simulation times are what we want them to be.
      lrRet = pConnection->NotifyPreBeginSim() ;
      break ;
   case PSAPIN_BEGINSIM:
      pConnection->m_State = SimRunning ;
      lrRet = pConnection->NotifyBeginSim() ;
      break ;
   case PSAPIN_ENDSIM:
      pConnection->m_State = SimStopped ;
      pConnection->NotifyEndSim() ;
      break ;
   case PSAPIN_PAUSEON:
      pConnection->m_bPaused = TRUE;
      pConnection->NotifyPauseOn() ;
      break ;
   case PSAPIN_PAUSEOFF:
      pConnection->m_bPaused = FALSE;
      pConnection->NotifyPauseOff() ;
      break ;
   case PSAPIN_WANTPAUSE: {   // BB960301: Added
      pConnection->m_dbTime = lpnds->uvs.dbTime ;
      pConnection->m_cValuesInTab = (int)((lpcds->cbData-16)/sizeof(double)) ;
      if (pConnection->m_cValuesInTab)
         pConnection->m_ValueTab = &lpnds->uvs.dbValueTab[0] ;
      lrRet = pConnection->NotifyWantPause() ;
      pConnection->m_cValuesInTab = 0 ;
      pConnection->m_ValueTab = NULL ;
      break ;
   } // case
   case PSAPIN_CALLBACK:
      lrRet = PSAPI_NotificationCallback(lpcds->lpData, lpcds->cbData) ;
      break ;
   default:
      ASSERT(0) ;
      break ;
   } // switch
   return lrRet ;
} // CPsConnection::Notify

int CPsConnection::GetModelCount() const
/* Purpose: Get the count of connected models.
   Return:  The requested number.
*/
{
	BOOL bOk = ((CPsConnection*)this)->VerifyOpen() ;
	int cModels = 0 ;
	if (bOk)
		cModels = PSAPI_GetModelCount(m_hConnection) ;
	return cModels ;
}


LPSTR CPsConnection::GetModelName(int iModel, LPSTR lpszName, UINT cbNameBuf) const
/* Purpose: Get the name of one of the connected models.
   In:      iModel - zero based index of the model.
	Out:     lpszName[cbNameBuf] - the name of the model.
	Return   lpszName
*/
{
	BOOL bOk = ((CPsConnection*)this)->VerifyOpen() ;
	if (bOk)
		PSAPI_GetModelName(m_hConnection, iModel, lpszName, cbNameBuf) ;
	else
		lstrcpyn(lpszName, "", cbNameBuf) ;
	return lpszName ;
}


int CPsConnection::GetVariableCount(int iModel) const
{
/* Purpose: Get the variable count in one of the connected models.
   In:      iModel - zero based index of the model.
	Return   Count
*/
	BOOL bOk = ((CPsConnection*)this)->VerifyOpen() ;
	int cVars = 0 ;
	if (bOk)
		cVars = PSAPI_GetVariableCount(m_hConnection, iModel) ;
	return cVars ;
}


LPSTR CPsConnection::GetVariableName(int iModel, int iVariable, 
	LPSTR lpszName, UINT cbNameBuf)
/* Purpose: Get the name of a variable in one of the connected models.
   In:      iModel - zero based index of the model.
	         iVariable - zero based index of the variable
	Out:     lpszName[cbNameBuf] - the name of the variable.
	Return   lpszName
*/
{
	BOOL bOk = ((CPsConnection*)this)->VerifyOpen() ;
	if (bOk)
		PSAPI_GetVariableName(m_hConnection, iModel, iVariable, 
			lpszName, cbNameBuf) ;
	else
		lstrcpyn(lpszName, "", cbNameBuf) ;
	return lpszName ;
}


BOOL CPsConnection::VariableHasInputs(int iModel, int iVariable) const
/* Purpose: Check whether a variable has inputs or not. Variables with
            inputs cannot be modified during initialization of model.
            The reason for this is that they must be updated if any
            of their inputs changes.
   In:      iModel - zero based index of the model.
	         iVariable - zero based index of the variable
   Return:  TRUE if the variable has inputs or if an error occurs,
            otherwise FALSE.
*/
{
	BOOL bOk = ((CPsConnection*)this)->VerifyOpen() ;
	BOOL bRet = FALSE ;
	if (bOk)
		bRet = (PSAPI_VariableHasInputs(m_hConnection, iModel, iVariable)!=0) ;
	return bRet ;
}


int CPsConnection::GetVariableType(int iModel, int iVariable) const
/* Purpose:    Check whether a variable in a model is dependent of other variables.
   In:         iModel - the index of the model to retrieve information from.
               iVariable - the variable to retrieve information about
   Return:     PSAPI_VT_LEVEL - level.
               PSAPI_VT_CONST - constant
               PSAPI_VT_AUX - axuxiliary
					0 - Error
*/
{
	BOOL bOk = ((CPsConnection*)this)->VerifyOpen() ;
	int Type = 0 ;
	if (bOk)
		Type = PSAPI_GetVariableType(m_hConnection, iModel, iVariable) ;
	return Type ;
}


LPSTR CPsConnection::GetElementName(int iModel, int iVariable, 
	int iElement, LPSTR lpszName, UINT cbNameBuf) const
/* Purpose:    Get the name of one of the elements of an array variable.
   In:         iModel - the index of the model. (0 is main model)
               iVariable - the zero based index of the variable.
               iElement - the zero based index of the element.
               cbNameBuf - size of buffer
   Out:        lpszName[] - buffer to retrieve name.
   Return:     lpszName
   Changes:    Who      When     What
               AK       961204   Created
*/
{
	BOOL bOk = ((CPsConnection*)this)->VerifyOpen() ;
	if (bOk)
		PSAPI_GetElementName(m_hConnection, iModel, iVariable, 
			iElement, lpszName, cbNameBuf) ;
	else
		lstrcpyn(lpszName, "", cbNameBuf) ;
	return lpszName ;
}


int CPsConnection::GetElementCount(int iModel, int iVariable) const
/* Purpose:    Get the total number of elements in a variable.
   In:         iModel - the index of the model to retrieve information from.
               iVariable - the variable to retrieve information about
   Return:     The number of elements if the variable is an array, 0 if it is
               a scalar or -1 if an error occurred.
   Changes:    Who      When     What
               AK       961204   Created
*/
{
	BOOL bOk = ((CPsConnection*)this)->VerifyOpen() ;
	int cElm = -1 ;
	if (bOk)
		cElm = PSAPI_GetElementCount(m_hConnection, iModel, iVariable) ;
	return cElm ;
}


BOOL CPsConnection::NotifyPreBeginSim() { return FALSE ; } // AK951019: Added
BOOL CPsConnection::NotifyBeginSim() { return FALSE ; }
void CPsConnection::NotifyEndSim() {}
void CPsConnection::NotifyOpenModel() {}  // OS950809: Added
void CPsConnection::NotifyCloseModel() {}
void CPsConnection::NotifyUpdateValues(BOOL) {}
void CPsConnection::NotifyParamDeleted(int) {}
void CPsConnection::NotifyPauseOn() {}
void CPsConnection::NotifyPauseOff() {}
BOOL CPsConnection::NotifyWantPause() { return FALSE;}

